// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef IOTHUBTRANSPORTMQTT_WEBSOCKETS_H
#define IOTHUBTRANSPORTMQTT_WEBSOCKETS_H

#include "iothub_transport_ll.h"

#ifdef __cplusplus
extern "C"
{
#endif

    extern const TRANSPORT_PROVIDER* MQTT_WebSocket_Protocol(void);

#ifdef __cplusplus
}
#endif

#endif /*IOTHUBTRANSPORTMQTT_WEBSOCKETS_H*/
