// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

/** @file iothub_client_version.h
*    @brief Functions for managing the client SDK version.
*/

#ifndef IOTHUB_CLIENT_PROVISIONING_H
#define IOTHUB_CLIENT_PROVISIONING_H

#include "azure_macro_utils/macro_utils.h"
#include "umock_c/umock_c_prod.h"

#ifdef USE_PROV_MODULE
#include "azure_prov_client/internal/iothub_auth_client.h"
#endif

#include "iothub_client_ll.h"

/*This header is DEPREPCATED and mainatined only for backward compatibility*/

#endif // IOTHUB_CLIENT_PROVISIONING_H
