



if not exist %PREFIX%\\Library\\bin\\iothub_client_dll.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\iothub_service_client.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\serializer.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
